/* $Id: main.c,v 1.5 2000/09/11 00:01:55 bodo Exp $ */
/*
 * This file is part of relay.
 *
 * Copyright (C) 2000 Bodo Bellut
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   Authors:  Bodo Bellut (bodo@bellut.net)
 */

#include <config.h>

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>

#include <relay.h>
#include <options.h>

int main (int argc, char *argv[]) {
 int sfd,num; 
 options opt;

 process_options(argc, argv, &opt);

 if ((sfd=open(opt.port,O_RDWR|O_NOCTTY))==-1) {
  perror("open");
  return -1;
 }
 num= initcards(sfd,1);
 
 if (!num) {
   fprintf(stderr, "No card found, exiting.\n");
   return -1;
 }
 
 if (opt.card > num) {
   fprintf(stderr, "Card %d does not exist, only %d card%s found.\n",
           opt.card, num, num > 1 ? "s" : "");
   return -1;
 }
 
 if (opt.clearall)
   printf("clearall-> %d\n", setrelay(sfd, opt.card, 0));
 
 if (opt.setall)
   printf("setall->   %d\n", setrelay(sfd, opt.card, 255));
 
 if (opt.clearexec || opt.clearblock || opt.setexec || opt.setblock) {
   int oldopt = getoption(sfd, opt.card);
   
   if (opt.clearexec)  oldopt &= ~ OPTMASK_EXEC;
   if (opt.setexec)    oldopt |=   OPTMASK_EXEC;
   if (opt.clearblock) oldopt &= ~ OPTMASK_BLOCK;
   if (opt.setblock)   oldopt |=   OPTMASK_BLOCK;
   
   printf("setoption->%d\n", setoption(sfd, opt.card, oldopt));
 }
 
 if (opt.numerically && opt.setrelay < 0)
   printf("0x%02x\n", getrelay(sfd, opt.card));
   
 if (opt.showstatus) {
   int status = getrelay(sfd, opt.card);
   int i;

   for (i=0; i < 8; i++)
     printf("relay %d: %s\n", i+1, ((status >> i) & 1) ? "on" : "off");
 }
 
 if (opt.showopts) {
   int opts = getoption(sfd, opt.card);
   fprintf(stderr, "DEBUG> opts = 0x%02x\n", opts);
   fflush(stderr);
   printf("execute broadcasts: %s\n", (opts & OPTMASK_EXEC)  ? "yes" : "no");
   printf("block   broadcasts: %s\n", (opts & OPTMASK_BLOCK) ? "yes" : "no");
 }
   
 if (opt.numerically && opt.setrelay >= 0)
   printf("setrelay-> %d\n", setrelay(sfd, opt.card, opt.setrelay));
 
 if (opt.individual)
 {
 int i;
 int status = getrelay(sfd, opt.card);
 
 for (i=0; i < 8; i++) {
   status |=    opt.set[i]   << i;
   status &= ~ (opt.clear[i] << i);
 }
 
 printf("setrelay-> %d\n", setrelay(sfd, opt.card, status));
 
 }


 restoretty(sfd);
 close(sfd);
 
 return 0;
 
}
